
# IOTConnect SDK: iotconnect-java-sdk

This is an JAVA SDK library to connect the device with IoTConnect cloud by MQTT protocol. This library only abstract JSON responses from both end D2C and C2D. This SDK supports SAS key based authentication, CA signed and Self signed certificate authentication to communicate with cloud.

## Features:

* The SDK supports to send telemetry data and receive commands from IoTConnect portal.
* User can update firmware Over The Air using "OTA update" Feature supported by SDK.
* SDK support SAS authentication as well as x509 certificate authentication.  
* SDK consists of Gateway device with multiple child devices support.
* SDK supports to receive and update the Twin property. 
* SDK supports device and OTA command Acknowledgement.
* Edge device support with data aggregation.
* Provide device connection status receive by command.
* Support hard stop command to stop device client from cloud.
* It allows sending the OTA command acknowledgment for Gateway and child device.
* It manages the sensor data sending flow over the cloud by using data frequency("df") configuration.
* It allows to disconnect the device from firmware.

# Example Usage:

Follow the steps given below to import our SDK as a library
 - Open Eclipse IDE and Create/Open Project
 - Create new lib folder inside project
 - Add **jar** file into lib folder (Get **jar** file from “iotconnect-sdk-java-v3.0.zip”)


Prerequisite input data
```java
String uniqueId = << Device UniqueID >>;	//It's your device uniqueId
String cpid = << CPID >>;		       // Get your ENV and CPID from the portal key vaults module or visit https://help.iotconnect.io SDK section
String env = << Environment >>;
```

**SDKOptions** is for the SDK configuration and need to parse in SDK object initialize call. You need to manage the below configuration as per your device authentications.

```java
SDKOptions sdkOptions = new SDKOptions();
//For SSL CA signed and SelfSigned authorized device only
sdkOptions.setSSLCaPath("<<filepath>>/ca.cert.pem");
sdkOptions.setSSLCertPath("<<filepath>>/cert.pem");
sdkOptions.setSSLKeyPath("<<filepath>>/key.pem");
//For Offline Storage only
sdkOptions.setAvailSpaceInMb(<<Integer Value>>); //size in MB, Default value = unlimited
sdkOptions.setFileCount(<<Integer Value>>); // Default value = 1
sdkOptions.setDisabledOfflineStorage(<<true/false>>); //default value = false, false = store data, true = not store data 
```
"certificate": It is indicated to define the path of the certificate file. Mandatory for X.509/SSL device CA signed or self-signed authentication type only.
	- SSLKeyPath: your device key
	- SSLCertPath: your device certificate
	- SSLCaPath : Root CA certificate
"offlineStorage" : Define the configuration related to the offline data storage 
	- disabled : false = offline data storing, true = not storing offline data 
	- availSpaceInMb : Define the file size of offline data which should be in (MB)
	- fileCount : Number of files need to create for offline data
**Note:**: sdkOptions is optional but mandatory for SSL/x509 device authentication type only. Define proper setting or leave it NULL. If you do not provide offline storage, it will set the default settings as per defined above. It may harm your device by storing the large data. Once memory gets full may chance to stop the execution.


To Initialize the SDK object and connect to the cloud
```java
SDKClient sdkClient = new SDKClientImpl(cpId, uniqueId, callerClassObject, callerClassObject, sdkOptions, env);
```

To receive the command from Cloud to Device(C2D)
```java
@Override
public void deviceCallback(D2CResponse d2CResponse) {
	try {
		ObjectMapper ackMapper = new ObjectMapper();
		if(d2CResponse != null && d2CResponse.getD2cData() != null){
			if(d2CResponse.getCmdType().equals("0x01")){        
				// Device Command
			}else if(d2CResponse.getCmdType().equals("0x02")){  
				// Firmware Command
			}else if(d2CResponse.getCmdType().equals("0x16")){
				// Device Connection status (command : true [connected] and command : false [disconnected])
			}
		}
	} catch (Exception e) {
		System.out.println("Error While Sending Acknowledgement to HUB"+ e.getMessage());
	}
}
```

To receive the twin from Cloud to Device(C2D)
```java
@Override
public void twinUpdateCallback(Map<String, Object> twinProperties) {
	if(twinProperties.size() > 0){
		System.out.println(twinProperties.toString());
	}
}
```

To get the list of attributes
```java
String attributes = sdkClient.getAttributes();
```

This is the standard data input format for Gateway and non Gateway device.
```java
1. For Non Gateway Device 
String deviceData = "[{'data':{'Humidity':'100'},'time':'2020-02-20T07:54:32.321Z','uniqueId':'<<uniqueId>>'}]";

2. For Gateway and multiple child device 
String deviceData = "[{'data':{'Temperature':'10'},'time':'2020-02-20T06:58:42.875Z','uniqueId':'<<uniqueId>>'},{'data':{'Gyroscope':{'x':'20','y':'30','z':'40'}},'time':'2020-02-20T06:58:51.224Z','uniqueId':'<<Child_UniqueID>>'}]";

Note: Date time will be in ISO 8601 format as "YYYY-MM-DDTHH:MM:SS.sssZ". Example: "2019-12-24T10:06:17.857Z"
```

To send the data from Device To Cloud(D2C)
```java
sdkClient.sendData(deviceData);
```

To send the command acknowledgment
```java
String ackMsg = "[{'data': {'ackId': '<<ackId>>','st': <<st value>>,'msg':'<custom message>','childId': '<gateway's child device Id>'},'uniqueId':'<<uniqueId>>'}]";

String msgType = ""; // 5 ("0x01" device command), 11 ("0x02" Firmware OTA command)
sdkClient.sendAck(ackMsg, msgType);
```
"ackId(*)" 	: Command Acknowledgment GUID which will receive from command payload (data.ackId)
"st(*)"		: Acknowledgment status sent to cloud (4 = Fail, 6 = Device command[0x01], 7 = Firmware OTA command[0x02])
"msg" 		: It is used to send your custom message
"childId" 	: It is used for Gateway's child device OTA update only
				0x01 : null or "" for Device command
			  	0x02 : null or "" for Gateway device and mandatory for Gateway child device's OTA update.
		   		How to get the "childId" .?
		   		- You will get child uniqueId for child device OTA command from payload "data.urls[~].uniqueId"
"msgType" 	: Message type (5 = "0x01" device command, 11 = "0x02" Firmware OTA command)
Note : (*) indicates the mandatory element of the object.

To update the Twin Property
```java
Map<String,Object> twinPropertyMap = new HashMap<>();
twinPropertyMap.put("<< Desired property key >>", "<< Desired Property value >>"); // Desired property key and value received from twinUpdateCallback method

Example :	
Map<String,Object> twinPropertyMap = new HashMap<>();
twinPropertyMap.put("humidity", "100");
sdkClient.updateTwin(twinPropertyMap);
```
"key" 	:	Desired property key received from Twin callback message
"value"	:	Value of the respective desired property

To disconnect the device from the cloud
```java
sdkClient.dispose();
```

To get the all twin property Desired and Reported
```java
sdkClient.getAllTwins();
```

To get the all attributes 
```java
sdkClient.getAttributes();
```

# Dependencies:

* This SDK used below packages :
	- mqtt-client, org.eclipse.paho.client.mqttv3, geronimo-jms_1.1_spec, gson, jackson-databind, httpclient, iot-device-client, amqp-client, quartz, slf4j-log4j12, bcprov-jdk16, commons-io

# Integration Notes:

## Prerequisite tools

1. Java 8
2. Eclipse IDE

## Installation:

* Extract the "iotconnect-sdk-java-v3.0.zip"

* Follow the steps given below to import our SDK as a library
	- Open Eclipse IDE and Create/Open Project
	- Create new lib folder inside project
	- Add **jar** file into lib folder (Get **jar** file from “iotconnect-sdk-java-v3.0.zip”)
	
* You can take the firmware file from the above location and update the following details
    - Prerequisite input data as explained in the usage section as above
    - Update sensor attributes according to added in IoTConnect cloud platform.
    - If your device is secure then need to configure the x.509 certificate path such as given above in SDK Options otherwise leave it as it is.

* Ready to go:
    - Click the "Run" button in the toolbar in Eclipse.
    
## Release Note :

** New Feature **
1. Offline data storage functionality with specific settings.
2. Edge enable device support Gateway device too.
3. Device and OTA command acknowledgment.
4. It allows to disconnecting the device client. 
5. Introduce new methods:
	sendAck() : to send the command acknowledgement to cloud
	dispose() : to disconnect the device
	getAllTwins() : To receive all the twin properties
6. Support hard stop command from cloud.
7. Support OTA command with Gateway and child device.
8. It allows sending the OTA command acknowledgment for Gateway and child device.
9. Introduce new command(0x16) in device callback for Device connection status true(connected) or false(disconnected).

** Improvements **
1. We have updated below methods name:
    Update Package Name:
	- Old : package sdk;
	- New : package iotconnect.sdk;
	Update Class Name:
	- Old : IoTConnectSDK
	- New : SDKClient
	Update Class Name:
	- Old : IoTConnectSDKImpl
	- New : SDKClientImpl
	Update Method Name:
	- Old : IoTConnectSDKImpl
	- New : SDKClientImpl
   To Initialize the SDK object:
	- Old : new IoTConnectSDKImpl(cpId, uniqueId, env, cb, cb);
	- New : new SDKClientImpl(cpId, uniqueId, env, cb, cb);
	- New : new SDKClientImpl(cpId, uniqueId, env, cb, cb, sdkOptions);  // Override SDK Configuration
   To receive Device command callback :
    - Old : sdkCallbackMethod(D2CResponse d2CResponse);
	- New : deviceCallback(D2CResponse d2CResponse);
   To receive OTA command callback :
    - Old : twinCallBackMethod(Map<String, Object> twinProperties);
	- New : twinUpdateCallback(Map<String, Object> twinProperties);
2. Remove config.properties file and use the SDKOptions for the certificate and offline data storage configuration.
3. Update the OTA command receiver payload for multiple OTA files.
4. Use the "df" Data Frequency feature to control the flow of data which publish on cloud (For Non-Edge device only).